#if !defined(__linux__) && !defined(_WIN32)

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <string.h>

#else

#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <fcntl.h>

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;

#endif

/* F, G, H and I are basic MD5 functions.
 */
static inline int16_t F(int16_t x, int16_t y, int16_t z) {
	return (x & y) | (~x & z);
}

static inline int16_t G(int16_t x, int16_t y, int16_t z) {
	return (x & z) | (y & ~z);
}

static inline int16_t H(int16_t x, int16_t y, int16_t z) {
	return (x ^ y ^ z);
}

static inline int16_t I(int16_t x, int16_t y, int16_t z) {
	return (y ^ (x | ~z));
}

static inline int16_t ROTATE_LEFT(int16_t x, int16_t n) {
	return (x << n) | ((uint16_t)x >> (16-n));
}
/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
Rotation is separate from addition to prevent recomputation.
 */
static inline void FF(int16_t *state, int16_t a, int16_t b, int16_t c,int16_t d,int16_t x,int16_t s,int16_t ac) {
 state[a] += F (state[b], state[c], state[d]) + (x) + (ac); 
 state[a] = ROTATE_LEFT (state[a], (s)); 
 state[a] += state[b]; 
}

static inline void GG(int16_t *state, int16_t a, int16_t b, int16_t c,int16_t d,int16_t x,int16_t s,int16_t ac) {
 state[a] += G (state[b], state[c], state[d]) + (x) + (ac); 
 state[a] = ROTATE_LEFT (state[a], (s)); 
 state[a] += state[b]; 
}

static inline void HH(int16_t *state, int16_t a, int16_t b, int16_t c,int16_t d,int16_t x,int16_t s,int16_t ac) {
 state[a] += H (state[b], state[c], state[d]) + (x) + (ac); 
 state[a] = ROTATE_LEFT (state[a], (s)); 
 state[a] += state[b]; 
}

static inline void II(int16_t *state, int16_t a, int16_t b, int16_t c,int16_t d,int16_t x,int16_t s,int16_t ac) {
 state[a] += I (state[b], state[c], state[d]) + (x) + (ac); 
 state[a] = ROTATE_LEFT (state[a], (s)); 
 state[a] += state[b]; 
}

/* Constants for MD5Transform routine.
 */
static const int S11 = 7;
static const int S12 = 12;
static const int S13 = 17;
static const int S14 = 22;
static const int S21 = 5;
static const int S22 = 9;
static const int S23 = 14;
static const int S24 = 20;
static const int S31 = 4;
static const int S32 = 11;
static const int S33 = 16;
static const int S34 = 23;
static const int S41 = 6;
static const int S42 = 10;
static const int S43 = 15;
static const int S44 = 21;

void miniD5Transform (int16_t *state, int16_t *x) {
  int16_t a=0,b=1,c=2,d=3;
  int16_t old[4];
//  int16_t x[16];
  old[0] = state[0];
  old[1] = state[1];
  old[2] = state[2];
  old[3] = state[3];

//  Decode (x, block, ioff, 64);

  /* Round 1 */
  FF (state, a, b, c, d, x[ 0], S11, 0xa478); /* 1 */
  FF (state, d, a, b, c, x[ 1], S12, 0xb756); /* 2 */
  FF (state, c, d, a, b, x[ 2], S13, 0x70db); /* 3 */
  FF (state, b, c, d, a, x[ 3], S14, 0xceee); /* 4 */

 /* Round 2 */
  GG (state, a, b, c, d, x[ 1], S21, 0x2562); /* 17 */
  GG (state, d, a, b, c, x[ 6], S22, 0xb340); /* 18 */
  GG (state, c, d, a, b, x[11], S23, 0x5a51); /* 19 */
  GG (state, b, c, d, a, x[ 0], S24, 0xc7aa); /* 20 */

  /* Round 3 */
  HH (state, a, b, c, d, x[ 5], S31, 0x3942); /* 33 */
  HH (state, d, a, b, c, x[ 8], S32, 0xf681); /* 34 */
  HH (state, c, d, a, b, x[11], S33, 0x6122); /* 35 */
  HH (state, b, c, d, a, x[14], S34, 0x380c); /* 36 */

  /* Round 4 */
  II (state, a, b, c, d, x[ 0], S41, 0x2244); /* 49 */
  II (state, d, a, b, c, x[ 7], S42, 0xff97); /* 50 */
  II (state, c, d, a, b, x[14], S43, 0x23a7); /* 51 */
  II (state, b, c, d, a, x[ 5], S44, 0xa039); /* 52 */

  state[0] += old[0];
  state[1] += old[1];
  state[2] += old[2];
  state[3] += old[3];

}
