`timescale 10ps / 10ps
`define GTK // use VCD

`include "rtavr_defs.v"

`define MODULE	tb_int0
`define CLK_LIMIT	10000

module `MODULE;

   reg [15:0] CLK_COUNT;
   reg CLK2X;  // output
   reg RESET;  // output
   wire CLK;   // input
   reg PC2;    // 

  `ifdef RTAVR_PORT_ESCALATION
     wire [23:0] DDR;
     wire [23:0] PORT;
     reg [23:0] PIN;
  `endif

rtavr top (.RESET(RESET), .CLK2X(CLK2X)
		, .CLK(CLK)
  `ifdef RTAVR_PORT_ESCALATION
		, .DDR(DDR[23:0])
		, .PORT(PORT[23:0])
		, .PIN(PIN[23:0])
  `else
    `ifdef IOR_HAVE_PORTA
		, .PORTA(PORTA[7:0])
    `endif
    `ifdef IOR_HAVE_PORTB
		, .PORTB(PORTB[7:0])
    `endif
    `ifdef IOR_HAVE_PORTC
		, .PORTC(PORTC[7:0])
    `endif
  `endif
 );

   parameter CLKP = 5000;        //  50ns
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK2X = 0;
      #CLKL CLK2X = 1;
   end

   always @(negedge CLK)
   begin
	 if (DDR[16+2]) PIN[16+2] = PORT[16+2]; // PC2 (INT0)
	 if (DDR[16+2]) PC2 = PORT[16+2]; // PC2 (INT0)
   end

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE.top);
   end
`endif

   initial begin : test
      sys_reset();
      while ( CLK_COUNT < `CLK_LIMIT)
      begin
         @(posedge CLK2X);
         @(posedge CLK2X);
         CLK_COUNT = CLK_COUNT + 1;
      end
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      begin
         PC2 = 0;
         CLK_COUNT = 0;
         #1 RESET = 0;
         CLK2X = 0;
         @(posedge CLK2X);
         @(posedge CLK2X);
	 @(negedge CLK2X);
         @(posedge CLK2X);
         #1 RESET = 1;
         @(posedge CLK2X);
         @(posedge CLK2X);
         CLK_COUNT = CLK_COUNT + 1;
      end
   endtask // sys_reset
   
endmodule // test
